@@ -1,5 +1,4 @@
-#define DNS_USE_GETTIMEOFDAY_FOR_ID 1
-#define HAVE_STRUCT_IN6_ADDR 1
+/* $Id: evdns.c,v 1.16 2007-11-06 16:09:37 root Exp $ */
 
 /* The original version of this module was written by Adam Langley; for
  * a history of modifications, check out the subversion logs.
@@ -41,8 +40,10 @@
 #endif
 
 #ifdef WIN32
+#ifndef EV_STANDALONE
 #include "misc.h"
 #endif
+#endif
 
 /* #define NDEBUG */
 
@@ -79,7 +80,6 @@
 
 #include <string.h>
 #include <fcntl.h>
-#include <sys/time.h>
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
 #endif
@@ -87,7 +87,6 @@
 #include <string.h>
 #include <errno.h>
 #include <assert.h>
-#include <unistd.h>
 #include <limits.h>
 #include <sys/stat.h>
 #include <ctype.h>
@@ -2547,7 +2546,7 @@ evdns_resolv_set_defaults(int flags) {
 
 #ifndef HAVE_STRTOK_R
 static char *
-fake_strtok_r(char *s, const char *delim, char **state) {
+strtok_r(char *s, const char *delim, char **state) {
 	return strtok(s, delim);
 }
 #endif
@@ -2623,9 +2622,9 @@ static void
 resolv_conf_parse_line(char *const start, int flags) {
 	char *strtok_state;
 	static const char *const delims = " \t";
-#define NEXT_TOKEN fake_strtok_r(NULL, delims, &strtok_state)
+#define NEXT_TOKEN strtok_r(NULL, delims, &strtok_state)
 
-	char *const first_token = fake_strtok_r(start, delims, &strtok_state);
+	char *const first_token = strtok_r(start, delims, &strtok_state);
 	if (!first_token) return;
 
 	if (!strcmp(first_token, "nameserver") && (flags & DNS_OPTION_NAMESERVERS)) {
