@@ -31,18 +31,30 @@
 
 #include <stddef.h>
 #include <stdlib.h>
-#include <sys/time.h>
+#include <assert.h>
 
+#ifndef WIN32
+# include <sys/time.h>
+#endif
+
+#include "ev.h"
 #include "event.h"
 
+#if EV_MULTIPLICITY
+# define dLOOPev struct ev_loop *loop = (struct ev_loop *)ev->ev_base
+# define dLOOPbase struct ev_loop *loop = (struct ev_loop *)base
+#else
+# define dLOOPev
+# define dLOOPbase
+#endif
+
+/* never accessed, will always be cast from/to ev_loop */
 struct event_base
 {
   int dummy;
 };
 
-static int x_actives;
-
-static struct event_base x_base, *x_cur;
+static struct event_base *x_cur;
 
 static void
 tv_set (struct timeval *tv, ev_tstamp at)
@@ -74,15 +86,28 @@ const char *event_get_method (void)
 
 void *event_init (void)
 {
-  if (ev_init (0))
-    return x_cur = &x_base;
+#if EV_MULTIPLICITY
+  if (x_cur)
+    x_cur = (struct event_base *)ev_loop_new (EVMETHOD_AUTO);
+  else
+    x_cur = (struct event_base *)ev_default_loop (EVMETHOD_AUTO);
+#else
+  assert (("multiple event bases not supported when not compiled with EV_MULTIPLICITY", !x_cur));
 
-  return 0;
+  x_cur = (struct event_base *)(long)ev_default_loop (EVMETHOD_AUTO);
+#endif
+
+  return x_cur;
 }
 
 void event_base_free (struct event_base *base)
 {
-  /* nop */
+  dLOOPbase;
+
+#if EV_MULTIPLICITY
+  if (ev_default_loop (EVMETHOD_AUTO) != loop)
+    ev_loop_destroy (loop);
+#endif
 }
 
 int event_dispatch (void)
@@ -90,10 +115,12 @@ int event_dispatch (void)
   return event_base_dispatch (x_cur);
 }
 
+#ifdef EV_STANDALONE
 void event_set_log_callback (event_log_cb cb)
 {
   /* nop */
 }
+#endif
 
 int event_loop (int flags)
 {
@@ -102,58 +129,55 @@ int event_loop (int flags)
 
 int event_loopexit (struct timeval *tv)
 {
-  event_base_loopexit (x_cur, tv);
+  return event_base_loopexit (x_cur, tv);
 }
 
 static void
 x_cb (struct event *ev, int revents)
 {
-  if (ev_is_active (&ev->sig))
-    {
-      ev_signal_stop (&ev->sig);
-      --x_actives;
-    }
-
-  if (!(ev->ev_events & EV_PERSIST) && ev_is_active (&ev->io))
-    {
-      ev_io_stop (&ev->io);
-      --x_actives;
-    }
-
   revents &= EV_READ | EV_WRITE | EV_TIMEOUT | EV_SIGNAL;
 
-  if (revents & EV_TIMEOUT)
-    --x_actives;
-
   ev->ev_res = revents;
   ev->ev_callback (ev->ev_fd, revents, ev->ev_arg);
 }
 
 static void
-x_cb_io (struct ev_io *w, int revents)
+x_cb_sig (EV_P_ struct ev_signal *w, int revents)
 {
-  x_cb ((struct event *)(((char *)w) - offsetof (struct event, io)), revents);
+  x_cb ((struct event *)(((char *)w) - offsetof (struct event, iosig.sig)), revents);
 }
 
 static void
-x_cb_to (struct ev_timer *w, int revents)
+x_cb_io (EV_P_ struct ev_io *w, int revents)
 {
-  x_cb ((struct event *)(((char *)w) - offsetof (struct event, to)), revents);
+  struct event *ev = (struct event *)(((char *)w) - offsetof (struct event, iosig.io));
+
+  if (!(ev->ev_events & EV_PERSIST) && ev_is_active (w))
+    ev_io_stop (EV_A_ w);
+
+  x_cb (ev, revents);
 }
 
 static void
-x_cb_sig (struct ev_signal *w, int revents)
+x_cb_to (EV_P_ struct ev_timer *w, int revents)
 {
-  x_cb ((struct event *)(((char *)w) - offsetof (struct event, sig)), revents);
+  struct event *ev = (struct event *)(((char *)w) - offsetof (struct event, to));
+
+  event_del (ev);
+
+  x_cb (ev, revents);
 }
 
 void event_set (struct event *ev, int fd, short events, void (*cb)(int, short, void *), void *arg)
 {
-  ev_watcher_init (&ev->io, x_cb_io);
-  ev_watcher_init (&ev->to, x_cb_to);
-  ev_watcher_init (&ev->sig, x_cb_sig);
+  if (events & EV_SIGNAL)
+    ev_watcher_init (&ev->iosig.sig, x_cb_sig);
+  else
+    ev_watcher_init (&ev->iosig.io, x_cb_io);
 
-  ev->ev_base     = x_cur;
+  ev_watcher_init (&ev->to, x_cb_to);
+
+  ev->ev_base     = x_cur; /* not threadsafe, but its like libevent works */
   ev->ev_fd       = fd;
   ev->ev_events   = events;
   ev->ev_pri      = 0;
@@ -164,48 +188,31 @@ void event_set (struct event *ev, int fd, short events, void (*cb)(int, short, v
 
 int event_once (int fd, short events, void (*cb)(int, short, void *), void *arg, struct timeval *tv)
 {
-  event_base_once (x_cur, fd, events, cb, arg, tv);
+  return event_base_once (x_cur, fd, events, cb, arg, tv);
 }
 
 int event_add (struct event *ev, struct timeval *tv)
 {
-  if (tv)
-    {
-      if (ev_is_active (&ev->to))
-        {
-          ev_timer_stop (&ev->to);
-          --x_actives;
-        }
+  dLOOPev;
 
-      ev_timer_set (&ev->to, tv_get (tv), 0.);
-      ev_timer_start (&ev->to);
-      ++x_actives;
-    }
-
-  if (ev->ev_events & (EV_READ|EV_WRITE))
-    {
-      if (ev_is_active (&ev->io))
-        {
-          ev_io_stop (&ev->io);
-          --x_actives;
-        }
-
-      ev_io_set (&ev->io, ev->ev_fd, ev->ev_events);
-      ev_io_start (&ev->io);
-      ++x_actives;
-    }
+  /* disable all watchers */
+  event_del (ev);
 
   if (ev->ev_events & EV_SIGNAL)
     {
-      if (ev_is_active (&ev->sig))
-        {
-          ev_signal_stop (&ev->sig);
-          --x_actives;
-        }
+      ev_signal_set (&ev->iosig.sig, ev->ev_fd);
+      ev_signal_start (EV_A_ &ev->iosig.sig);
+    }
+  else if (ev->ev_events & (EV_READ | EV_WRITE))
+    {
+      ev_io_set (&ev->iosig.io, ev->ev_fd, ev->ev_events & (EV_READ | EV_WRITE));
+      ev_io_start (EV_A_ &ev->iosig.io);
+    }
 
-      ev_signal_set (&ev->sig, ev->ev_fd);
-      ev_signal_start (&ev->sig);
-      ++x_actives;
+  if (tv)
+    {
+      ev_timer_set (&ev->to, tv_get (tv), 0.);
+      ev_timer_start (EV_A_ &ev->to);
     }
 
   return 0;
@@ -213,50 +220,54 @@ int event_add (struct event *ev, struct timeval *tv)
 
 int event_del (struct event *ev)
 {
-  if (ev_is_active (&ev->io))
+  dLOOPev;
+
+  if (ev->ev_events & EV_SIGNAL)
     {
-      ev_io_stop (&ev->io);
-      --x_actives;
+      /* sig */
+      if (ev_is_active (&ev->iosig.sig))
+        ev_signal_stop (EV_A_ &ev->iosig.sig);
+    }
+  else if (ev->ev_events & (EV_READ | EV_WRITE))
+    {
+      /* io */
+      if (ev_is_active (&ev->iosig.io))
+        ev_io_stop (EV_A_ &ev->iosig.io);
     }
 
   if (ev_is_active (&ev->to))
-    {
-      ev_timer_stop (&ev->to);
-      --x_actives;
-    }
-
-  if (ev_is_active (&ev->sig))
-    {
-      ev_signal_stop (&ev->sig);
-      --x_actives;
-    }
+    ev_timer_stop (EV_A_ &ev->to);
 
   return 0;
 }
 
-void event_active (struct event *ev, int fd, short events)
-{
-  /* unclear what this does, likely internal-use */
-}
-
 int event_pending (struct event *ev, short events, struct timeval *tv)
 {
-  short revents;
+  short revents = 0;
+  dLOOPev;
 
-  if (ev->io.pending)
-    revents |= ev->ev_events & (EV_READ | EV_WRITE);
 
-  if (ev->to.pending)
+  if (ev->ev_events & EV_SIGNAL)
+    {
+      /* sig */
+      if (ev_is_active (&ev->iosig.sig) || ev_is_pending (&ev->iosig.sig))
+        revents |= EV_SIGNAL;
+    }
+  else if (ev->ev_events & (EV_READ | EV_WRITE))
+    {
+      /* io */
+      if (ev_is_active (&ev->iosig.io) || ev_is_pending (&ev->iosig.io))
+        revents |= ev->ev_events & (EV_READ | EV_WRITE);
+    }
+
+  if (ev->ev_events & EV_TIMEOUT || ev_is_active (&ev->to) || ev_is_pending (&ev->to))
     {
       revents |= EV_TIMEOUT;
 
       if (tv)
-        tv_set (tv, ev_now); /* not sure if this is right :) */
+        tv_set (tv, ev_now (EV_A)); /* not sure if this is right :) */
     }
 
-  if (ev->sig.pending)
-    revents |= EV_SIGNAL;
-
   return events & revents;
 }
 
@@ -281,11 +292,9 @@ int event_base_set (struct event_base *base, struct event *ev)
 
 int event_base_loop (struct event_base *base, int flags)
 {
-  do
-    {
-      ev_loop (flags | EVLOOP_ONESHOT);
-    }
-  while (!(flags & (EVLOOP_ONESHOT | EVLOOP_NONBLOCK)) && x_actives && !ev_loop_done);
+  dLOOPbase;
+
+  ev_loop (EV_A_ flags);
 
   return 0;
 }
@@ -296,16 +305,21 @@ int event_base_dispatch (struct event_base *base)
 }
 
 static void
-x_loopexit_cb (int revents, void *arg)
+x_loopexit_cb (int revents, void *base)
 {
-  ev_loop_done = 2;
+  dLOOPbase;
+
+  ev_unloop (EV_A_ EVUNLOOP_ONCE);
 }
 
 int event_base_loopexit (struct event_base *base, struct timeval *tv)
 {
   ev_tstamp after = tv_get (tv);
+  dLOOPbase;
 
-  ev_once (-1, 0, after >= 0. ? after : 0., x_loopexit_cb, (void *)base);
+  ev_once (EV_A_ -1, 0, after >= 0. ? after : 0., x_loopexit_cb, (void *)base);
+
+  return -1;
 }
 
 struct x_once
@@ -327,6 +341,7 @@ x_once_cb (int revents, void *arg)
 int event_base_once (struct event_base *base, int fd, short events, void (*cb)(int, short, void *), void *arg, struct timeval *tv)
 {
   struct x_once *once = malloc (sizeof (struct x_once));
+  dLOOPbase;
 
   if (!once)
     return -1;
@@ -335,13 +350,15 @@ int event_base_once (struct event_base *base, int fd, short events, void (*cb)(i
   once->cb  = cb;
   once->arg = arg;
 
-  ev_once (fd, events & (EV_READ|EV_WRITE), tv_get (tv), x_once_cb, (void *)once);
+  ev_once (EV_A_ fd, events & (EV_READ | EV_WRITE), tv_get (tv), x_once_cb, (void *)once);
 
   return 0;
 }
 
 int event_base_priority_init (struct event_base *base, int npri)
 {
+  /*dLOOPbase;*/
+
   return 0;
 }
 
